## Copyright (c) Thomas Bock, 2018
## bockthom@fim.uni-passau.de
##
## Distributed under terms of the MIT license.

## libraries
suppressWarnings(library(plyr)) # tools for splitting, applying and combining data
suppressWarnings(library(lubridate)) # tools for handling dates
suppressWarnings(library(parallel)) # tools for parallel execution

do.qualitative.analysis.evaluation <- function(artefact, thread.based=TRUE) {

  only.pairs.having.more.than.x.c.e.bursts <- TRUE
  x = 0 # 5

  if(thread.based) {
    thread = "thread_"
  } else {
    thread = ""
  }
  file.name.devCEbursts <- paste0(PATH.TO.DATA, "/", "e_bursts_", thread, artefact,
                                  "_qualitative_analysis_", EPSILON, "_devCEbursts.dat")
  file.name.bursts.emails <- paste0(PATH.TO.DATA, "/", "e_bursts_", thread, "mbox_", artefact,
                                   "_qualitative_analysis_", EPSILON, ".dat")
  file.name.bursts.commit <- paste0(PATH.TO.DATA, "/", "c_bursts_qualitative_analysis_", artefact,
                                    "_", EPSILON, ".dat")

  load(file.name.devCEbursts)
  print(paste("Load file: ", file.name.devCEbursts))
  load(file.name.bursts.commit)
  print(paste("Load file: ", file.name.bursts.commit))
  load(file.name.bursts.emails)
  print(paste("Load file: ", file.name.bursts.emails))

  results <- mclapply(developer.pairs.c.and.e.bursts, mc.cores=CORES, function(developer.pair){

    if(only.pairs.having.more.than.x.c.e.bursts &&
       (developer.pair$email.bursts <= x || developer.pair$commit.bursts <= x)) {
      result <- NA
      return(result)
    }
    loginfo(paste(developer.pair$names[[1]], developer.pair$names[[2]]))
    commit.bursts <- burst.commits[[developer.pair$c.number]]
    email.bursts <- burst.emails[[developer.pair$e.number]]

    match.list <- c()

    for(cb in commit.bursts) {
      # get synchronously changed artefacts
      artefacts <- cb$artefacts
      found.match <- FALSE

      # get email bursts with a certain time difference
      allowed.ce.burst.distance <- duration(EPSILON, "day")

      for(eb in email.bursts) {
        if(!found.match &&
           ((min(ymd_hms(eb$emails$creationDate)) <= max(ymd_hms(cb$commits$authorDate))+allowed.ce.burst.distance) &&
           (max(ymd_hms(eb$emails$creationDate)) >= min(ymd_hms(cb$commits$authorDate))-allowed.ce.burst.distance))) {

          if(any(artefacts %in% eb$mboxparsing$artefact)) {
            loginfo(paste("CE match:", developer.pair$c.number, developer.pair$e.number))
            found.match = TRUE
          }
        }
      }

      match.list <- c(match.list, found.match)
    }

    result <- sum(match.list) / length(match.list)
    loginfo(paste("Result:", developer.pair$names[[1]], developer.pair$names[[2]], "mention-rate:", result))
    return(result)
  })

  if(only.pairs.having.more.than.x.c.e.bursts) {
    output.path.results <- paste0(PATH.TO.DATA, "/", "evaluation_qualCE_moreThan", x, "CE_",
                                  thread, artefact, "_", EPSILON, ".dat" )
  } else {
    output.path.results <- paste0(PATH.TO.DATA, "/", "evaluation_qualCE_",
                                  thread, artefact, "_", EPSILON, ".dat" )
  }
  save(results, file=output.path.results)

}

